import math
from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL import GLX
from OpenGL import GL
from ctypes import *
from point import createpoint


class createcamera:
    #your visible area in 2d space, cordinates of screen or window view space
    viewport = 400, 400
    viewport_aspect = viewport[0] / viewport[1]

    #how much of the world can we see, how wide is our sight
    field_of_view = 110

    #camera parameters position look direction rotation
    lookat = createpoint((0.0, 0.0, 100.0))
    location = createpoint((0.0, 0.0, 0.0))
    camera_rotation = 0.0

    #viewing depth of the camera, also know as the frustrum near and far planes
    near_plane = 10.0
    far_plane = 100.0

    #view frustrum / camera visible area properties,
    def __init__(self, width, height):
        self.viewport = width, height
        self.viewport_aspect = self.viewport[0] / self.viewport[1]

    def get_click_point(self, pos):
        """ convert 2d click in the viewport to 3d point in space"""
        viewport = glGetIntegerv(GL_VIEWPORT)
        modelview = glGetDoublev(GL_MODELVIEW_MATRIX)
        projection = glGetDoublev(GL_PROJECTION_MATRIX)

        #convert screen ccordinate to opengl cordinates, this means modifying the y axes only
        x, y = pos[0], self.viewport[0] - pos[1]

        #use unproject to calculate the point and store the resutl in a point object
        return createpoint(gluUnProject(x, y, 0.20, modelview, projection, viewport), (250, 0, 0))

    def draw(self):
        pass

    #the window has been resized so recalculate display
    #also used in initial setup of screen
    def update(self):
        """call this when ever the screen is updated,
        calculates what's visible and where the camera exists in space.
        also set what we are loking at in the world"""
        glViewport(0, 0, self.viewport[0], self.viewport[1])
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()

        #setup camera parameters field of view, size of camera (ie the view window size)
        #set the frustrum parameters
        gluPerspective(self.field_of_view,
                       1.0 * self.viewport[0] / self.viewport[1],
                       self.near_plane, self.far_plane)

        #position the camera and look at something
        gluLookAt(self.location.x, self.location.y, self.location.z,
                  self.lookat.x, self.lookat.y, self.lookat.z,
                  0.0, 1.0, 0.0)

        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
